package ve.gob.cenditel.tibisaymovil;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;

import ve.gob.cenditel.tibisaymovil.R;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.view.Window;
import android.webkit.MimeTypeMap;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.RadioButton;
import android.widget.TextView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.Toast;

public class FileToDecryptActivity extends Activity implements OnItemClickListener, OnClickListener {

	private File cwd;
    private File selected;
    private FileBrowserView viewHolder;
    private FileListAdapter listAdapter;
    private String filterMImeType = "application/*";
    
    // cadena que mantiene la ruta del archivo a compartir
    private String fileToDecrypt = null;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		//Estilando la barra de titulo
		final boolean customTitleSupported = requestWindowFeature(Window.FEATURE_CUSTOM_TITLE);

		super.onCreate(savedInstanceState);
		
		this.viewHolder = new FileBrowserView();	

        if (savedInstanceState != null) {
        	if(savedInstanceState.getString("selected") != null)
        		this.selected = new File(savedInstanceState.getString("selected"));

            if (this.selected != null) {
            	
            	FileToDecryptActivity.this.updateButton
            	(FileToDecryptActivity.this.viewHolder.accept,true);

            }
            
            this.cwd = new File(savedInstanceState.getString("cwd"));
            this.viewHolder.fileList.setAdapter(this.listAdapter = new FileListAdapter(this.cwd.getAbsolutePath(), filterMImeType));
            
        } else {
            this.selected = null;
            this.viewHolder.fileList.setAdapter(this.listAdapter = new FileListAdapter());
        }
        
        
        boolean enabled = false;
        if (this.selected != null)
        	enabled = this.viewHolder.accept.isEnabled();
        
        FileToDecryptActivity.this.updateButton
        (FileToDecryptActivity.this.viewHolder.accept,enabled);
        
        //Estilando Barra de titulo
		if(customTitleSupported)
			getWindow().setFeatureInt(Window.FEATURE_CUSTOM_TITLE, R.layout.title_bar);
	}

	
    /**
     * Provides the data to be shown in the file browser ListView.
     *
     * @author José M. Prieto (jmprieto@emergya.com)
     */
    private class FileListAdapter extends BaseAdapter {

        private final ArrayList<File> directories;
        private final ArrayList<File> files;

        private FileListAdapter() {
            this("/",filterMImeType);
        }

        private FileListAdapter(String location) {
            this(location, "");
        }
        
        private FileListAdapter(String location, String filterMimeType) {
        	
            directories = new ArrayList<File>();
            files = new ArrayList<File>();
            
            //Obtiene etiqueta que se colocará antes del path que visualizará el usuario
        	String toPathText = FileToDecryptActivity.this.getString(R.string.pathstring)+":   ";
        	//Coloca el texto de la etiqueta en la vista
        	FileToDecryptActivity.this.viewHolder.pathString.setText(toPathText);
        	//Coloca el texto con el path actual
        	FileToDecryptActivity.this.viewHolder.path.setText(location);
        	//Crea un objeto file cwd con la ubicacion dada en location
            FileToDecryptActivity.this.cwd = new File(location);
            //Obtiene el directorio padre del objeto file cwd
            File parent = FileToDecryptActivity.this.cwd.getParentFile();
            //Si tiene un padre, lo agrega en la posición cero de la lista de directorios  
            
            if (parent != null) {
                directories.add(0, parent);
            }
            
            //Crea un arreglo con las lista de archivos contenidos en el directorio cwd
            File[] ls = FileToDecryptActivity.this.cwd.listFiles();
            if (ls != null) {
                for (File f : ls) { //recorre todos los archivos contenidos en el directorio
                	
                    if (FsUtils.isHiddenOrNotReadable(f)) { // Si son ocultos no hace nada
                        continue;
                    }
                  // Si son directorios los agrega a la lista de directorios a partir de la posición 1
                  // En la posición 0 se encuentra el directorio padre  
                    if (f.isDirectory()) {
                        directories.add(f);
                    } else // De lo contrario lo agrega a la lista de archivos 
                    	{
                    	//Valida tipo de archivo a mostrar
                        Uri selectedUri = Uri.fromFile(f);
                        String fileExtension = MimeTypeMap.getFileExtensionFromUrl(selectedUri.toString());
                        String mimeType = MimeTypeMap.getSingleton().getMimeTypeFromExtension(fileExtension);
                          
//                          Toast.makeText(FileToDecryptActivity.this,
//                          "FileExtension: " + fileExtension + "\n" +
//                          "MimeType: " + mimeType,
//                          Toast.LENGTH_LONG).show();
                        
                                                  
                        // en principio no filtrar ningun archivo
                        if(filterMimeType.isEmpty() || filterMimeType == mimeType || fileExtension.equals("cdoc"))
                        	files.add(f);
                        	
                    }
                }
            }

            Collections.sort(directories); // Ordena los directorios alfabeticamente
            Collections.sort(files); // Ordena los archivos alfabeticamente
        }
       
        /**
         * Retorna cantidad total de elementos que se listarán en el directorio.
         */
        @Override 
        public int getCount() {
        	
            return directories.size() + files.size();
            
        }

        /**
         * Dada una posición en el listado del directorio, retorna un archivo o directorio
         * según corresponda.
         */
        @Override
        public File getItem(int position) {

            if (position < directories.size()) {
                return directories.get(position);
            } else {
                return files.get(position - directories.size());
            }
        }

        /**
         * Retorna un código hash para el archivo, permite comparar si dos archivos son los mismos
         */
        @Override
        public long getItemId(int position) {

            return getItem(position).hashCode();
        }
        
        /**
         * Crea la visualización de cada item del fileBrowser
         */
        @Override
        public View getView(int position, View convertView, ViewGroup parent) {

        	//Crea la vista de cada fila del filebrowser a partir del layout
            if (convertView == null) {
                LayoutInflater inflater = LayoutInflater.from(FileToDecryptActivity.this);
                convertView = inflater.inflate(R.layout.file_to_verify_bdoc_signature_item, parent, false); 
            }
            
            // Se enlaza a cada componente del layout 
            ImageView image = (ImageView) convertView.findViewById(R.id.type_image);
            TextView fileName = (TextView) convertView.findViewById(R.id.filename_text);
            TextView modified = (TextView) convertView.findViewById(R.id.filename_modified);
            
            // Se obtiene el archivo ubicado en position
        	File file = getItem(position);
        	
        	//RadioButton
            RadioButton radio = (RadioButton) convertView.findViewById(R.id.file_radio);
            radio.setFocusable(false);
            
            // Se asignan los iconos según el tipo de archivo y se oculta el radio en los directorios
            if (file.isDirectory()) {
                image.setImageResource(R.drawable.ic_carpeta);
                radio.setVisibility(View.INVISIBLE);
                radio.setChecked(false);
            } else {
                image.setImageResource(R.drawable.ic_archivo);
                radio.setVisibility(View.VISIBLE);
                
                if (FileToDecryptActivity.this.selected == null ||
	            	FileToDecryptActivity.this.selected.hashCode() != file.hashCode()){
	            		radio.setChecked(false);
            	} else{
            		radio.setChecked(true);
            	}         
            }

            // Si es el directorio que hace referencia al padre le coloca como nombre ".."
            if (file.isDirectory() && position == 0 && ! "/".equals(FileToDecryptActivity.this.cwd.getAbsolutePath())) {
                fileName.setText("..");
            } else {
                fileName.setText(file.getName());
            }
 
            
            //Datos de modificación del archivo
            SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
            Date date = new Date(file.lastModified());
            
            String dateString = FileToDecryptActivity.this.getString(R.string.modified)+": ";
            if (file.lastModified()>0)
            	modified.setText(dateString+sdf.format(date));
            else
            	modified.setText(dateString+"-");
            
            return convertView;
        }
        
        /**
         * Controla la selección de cada item del fileBrowser
         */
        public void select(ListView parent, int position) {

            File item = getItem(position);
            //Si es un directorio el seleccionado se hace un llamado del fileBrowser del directorio
            if (item.isDirectory()) {
                parent.setAdapter(FileToDecryptActivity.this.listAdapter = new FileListAdapter(item.getAbsolutePath(), filterMImeType));
            } else { // Si es un archivo
            	
        		//Se agrega el archivo a la lista de seleccionados si no se encuentra en la misma            	
            	if (FileToDecryptActivity.this.selected == null || 
            		FileToDecryptActivity.this.selected.hashCode() != item.hashCode()){
            		
            		FileToDecryptActivity.this.selected = item;
            		
            		FileToDecryptActivity.this.updateButton(FileToDecryptActivity.this.viewHolder.accept,true);  		
            		            		
            	}
            	else{ // De lo contrario se elimina de la lista de seleccionados
                	
            		FileToDecryptActivity.this.selected = null;             		
                	FileToDecryptActivity.this.updateButton(FileToDecryptActivity.this.viewHolder.accept,false);
                	               	
                }
                notifyDataSetChanged();
           }
        }       
    }    


    
    
    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);

        outState.putParcelable("intent", this.getIntent());
        outState.putString("cwd", this.cwd.getAbsolutePath());
        if(this.selected != null)
        	outState.putString("selected", this.selected.getAbsolutePath());
        
        }
    
    private void updateButton(View v, boolean bool) {
    	try{

        	v.setEnabled(bool);
        	if (v instanceof TextView){
        		Drawable icon = ((TextView)v).getCompoundDrawables()[1];
        		if (icon!=null)
        		if (bool)
        	   		icon.setAlpha(255);
        	   	else
        	   		icon.setAlpha(127);	
        	}
        	if (v instanceof ImageView){
        		ImageView result = (ImageView) v;
        		if (bool)
        			result.setAlpha(255);
        	   	else
        	   		result.setAlpha(127);	
        	}
        
    	}catch(NullPointerException e){}
	   		
	}
    
    private void updateButton(LinearLayout layout, boolean bool) {
    	try{
    	layout.setEnabled(bool);
        for (int i=0; i<layout.getChildCount();i++){
        	this.updateButton(layout.getChildAt(i), bool);
        }
    	}catch(NullPointerException e){}
	   		
	}

	
	
	/**
     * Holds references to view objects.
     *
     * @author José M. Prieto (jmprieto@emergya.com)
     */
    private class FileBrowserView {

        public ListView fileList;
        public TextView path;
        public LinearLayout accept;
		public LinearLayout clear;
        public TextView pathString;

        public FileBrowserView() {

            setContentView(R.layout.activity_file_to_decrypt);
            this.path = (TextView) findViewById(R.id.path);
            this.pathString = (TextView) findViewById(R.id.pathstring);

            this.fileList = (ListView) findViewById(R.id.file_list);
            this.fileList.setOnItemClickListener(FileToDecryptActivity.this);
            this.fileList.setItemsCanFocus(true);
         
            
            this.clear = (LinearLayout) findViewById(R.id.button_clear_zone);
            this.clear.setOnClickListener(FileToDecryptActivity.this);
            
            this.accept = (LinearLayout) findViewById(R.id.button_accept_zone);
            this.accept.setOnClickListener(FileToDecryptActivity.this);

        }
    }
    
	@Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
    	
    	
        if (parent.getId() == R.id.file_list) { // user selects a file
            this.listAdapter.select((ListView) parent, position);
            
        } else { // user de-selects a file
            //this.listAdapter.addIfSameDirectory(selectedAdapter.doUnselect((ListView) parent, position));
        	
        }
        //this.viewHolder.numSelected.setText(Integer.toString(this.selected.size()));
    }

    @Override
    public void onClick(View v) {

        switch (v.getId()) {

        
        case R.id.button_clear_zone:
        	
        	this.selected = null;
        	this.listAdapter.notifyDataSetChanged();

        	//this.viewHolder.numSelected.setText(""+this.selected.size());
            this.updateButton(this.viewHolder.accept, false);
        	break;        	
        

        
	    case R.id.button_accept_zone:
	    	
	    	// lanzar intent para compartir el archivo seleccionado
	    	fileToDecrypt = FileToDecryptActivity.this.selected.getAbsolutePath();
	    	
	    	Toast.makeText(getApplicationContext(), "FileToDecryptActivity: "+fileToDecrypt, Toast.LENGTH_SHORT).show();
	    	
	    		    	
	    	
	    	selectPKCS12ToDecrypt(fileToDecrypt);
	    						
	    	break;        	
    	
        }
    }
    
	
	// funcion para compartir el archivo
	private void shareIt() {
		
		Intent shareIntent = new Intent();
		shareIntent.setAction(Intent.ACTION_SEND);
		File file = new File(fileToDecrypt);
		
		
		Uri uri = Uri.fromFile(file);
		Log.i("DEBUG", file.getPath());
		//Log.d("******", getMimeType(file.getPath()));
		//shareIntent.setDataAndType(uri, getMimeType(file.getPath()));
		shareIntent.putExtra(Intent.EXTRA_STREAM, uri);
		shareIntent.setType("application/*");
		startActivity(Intent.createChooser(shareIntent, getResources().getText(R.string.share_it_using)));
	}
	
	
	/**
     * Selecciona el certificado del destinatario del directorio de certificados 
     * para cifrado 
     * @return void
     */
	// funcion para desplegar el gestor de certificados de destinatarios para cifrar
	private void selectPKCS12ToDecrypt(String fileToDecrypt) {
		
		// desplegar la actividad de gestor de certificados de destinatarios

		// chequear disponibilidad de directorio de certificados
	    if (!checkDecryptedDirectoryAvailability()){
	    	Toast.makeText(getApplicationContext(), "FileToDecryptActivity: directorio no disponible", Toast.LENGTH_SHORT).show();
	    	
	    	finish();
	    	return;
	    }else{
	    	// lanzar el activity SelectCeritificateToEncryptActivity
	    	Intent intent = new Intent(this, PKCS12ToDecryptActivity.class);		
			intent.putExtra("fileToDecrypt", fileToDecrypt);
			startActivity(intent);

	    }
		
		
		
		
		
	} // fin de selectPKCS12ToDecrypt()
	
	/**
     * Chequea la disponibilidad del directorio de /TibisayMovil/CertificatesToEncrypt
     * @return boolean
     */
    private boolean checkDecryptedDirectoryAvailability() {
    	// verificar acceso al directorio /mnt/sdcard/TibisayMovil/DecryptedFiles
	    boolean mExternalStorageAvailable = false;
	    boolean mExternalStorageWriteable = false;
	    String state = Environment.getExternalStorageState();
	    
	    String decryptedDir = Environment.getExternalStorageDirectory().getAbsolutePath() + "/" +
			getResources().getString(R.string.app_name) + "/" +
			getResources().getString(R.string.decrypted_dir_files) + "/";
	    
		AlertDialog.Builder builder = new AlertDialog.Builder(FileToDecryptActivity.this);
		
	    
	    if (Environment.MEDIA_MOUNTED.equals(state)) {
	        // We can read and write the media
	        mExternalStorageAvailable = mExternalStorageWriteable = true;
	        Toast.makeText(getApplicationContext(), "We can read and write the media", Toast.LENGTH_SHORT).show();
	        
	        // Crear directorio CertificatesToEncrypt donde se almacenan los certificados de
	        // destinatarios para cifrado
	        /*
			String decryptedDir = Environment.getExternalStorageDirectory() + "/" +
  			getResources().getString(R.string.app_name) + "/" +
  			getResources().getString(R.string.certificates_dir) + "/";
  			*/			
			if (prepareDirectory(decryptedDir)){				
				return true;
			}else{
				builder.setMessage("No existe el directorio "+decryptedDir+" para almacenar certificados.").setTitle("Error:");
        	
			}
			
	    } else if (Environment.MEDIA_MOUNTED_READ_ONLY.equals(state)) {
	        // We can only read the media
	        mExternalStorageAvailable = true;
	        mExternalStorageWriteable = false;
	        Toast.makeText(getApplicationContext(), "We can only read the media", Toast.LENGTH_SHORT).show();
	        
	        builder.setMessage("Directorio "+decryptedDir+ " montado de solo lectura. No se pueden almancenar certificados.").setTitle("Error:");
       	
	        
	    } else {
	        // Something else is wrong. It may be one of many other states, but all we need
	        //  to know is we can neither read nor write
	        mExternalStorageAvailable = mExternalStorageWriteable = false;
	        Toast.makeText(getApplicationContext(), "we can neither read nor write", Toast.LENGTH_SHORT).show();
	        
	        builder.setMessage("Directorio "+decryptedDir+ " no está disponible. No se pueden almancenar certificados.").setTitle("Error:");       		
       	
	    }
	    
	    
    	builder.setNegativeButton("Cancelar", new DialogInterface.OnClickListener() {
    		public void onClick(DialogInterface dialog, int id) {
               // User cancelled the dialog
    			FileToDecryptActivity.this.finish();
            }
    	});
	    AlertDialog dialog = builder.create();
	    dialog.show();
	    return false;
    } // fin de checkdecryptedDirectoryAvailability
	
    
    /**
     * Prepara directorio
     * @return boolean
     */
    boolean prepareDirectory(String dir) 
    {
        try
        {
            if (makedirs(dir)) 
            {
                return true;
            } else {
                return false;
            }
        } catch (Exception e) 
        {
            e.printStackTrace();
            Toast.makeText(this, "Could not initiate File System.. Is Sdcard mounted properly?", Toast.LENGTH_LONG).show();
            return false;
        }
    }
    
    /**
     * Crea directorio utilizando la variable tmpDir
     * @return boolean
     */
    private boolean makedirs(String dir) 
    {
        //File tempdir = new File(extractedDirFiles);
        File tempdir = new File(dir);
        if (!tempdir.exists())
            tempdir.mkdirs();
        return (tempdir.isDirectory());
    }
    
    
}

